@@ -25,7 +25,8 @@
  *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
-  
+
+#include "pcm_local.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -86,7 +87,12 @@ static void share_areas(snd_pcm_direct_t *dshare,
 	channels = dshare->channels;
 	format = dshare->shmptr->s.format;
 	if (dshare->interleaved) {
-		unsigned int fbytes = snd_pcm_format_physical_width(format) / 8;
+		int width = snd_pcm_format_physical_width(format);
+		unsigned int fbytes;
+
+		if (width < 0)
+			return;
+		fbytes = width / 8;
 		memcpy(((char *)dst_areas[0].addr) + (dst_ofs * channels * fbytes),
 		       ((char *)src_areas[0].addr) + (src_ofs * channels * fbytes),
 		       size * channels * fbytes);
@@ -110,7 +116,7 @@ static void snd_pcm_dshare_sync_area(snd_pcm_t *pcm)
 	snd_pcm_uframes_t slave_hw_ptr, slave_appl_ptr, slave_size;
 	snd_pcm_uframes_t appl_ptr, size;
 	const snd_pcm_channel_area_t *src_areas, *dst_areas;
-	
+
 	/* calculate the size to transfer */
 	size = pcm_frame_diff(dshare->appl_ptr, dshare->last_appl_ptr, pcm->boundary);
 	if (! size)
@@ -199,25 +205,17 @@ static int snd_pcm_dshare_sync_ptr0(snd_pcm_t *pcm, snd_pcm_uframes_t slave_hw_p
 static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
+	snd_pcm_uframes_t slave_hw_ptr;
 	int err;
 
-	switch (snd_pcm_state(dshare->spcm)) {
-	case SND_PCM_STATE_DISCONNECTED:
-		dshare->state = SNDRV_PCM_STATE_DISCONNECTED;
-		return -ENODEV;
-	case SND_PCM_STATE_XRUN:
-		if ((err = snd_pcm_direct_slave_recover(dshare)) < 0)
-			return err;
-		break;
-	default:
-		break;
-	}
-	if (snd_pcm_direct_client_chk_xrun(dshare, pcm))
-		return -EPIPE;
 	if (dshare->slowptr)
 		snd_pcm_hwsync(dshare->spcm);
+	slave_hw_ptr = *dshare->spcm->hw.ptr;
+	err = snd_pcm_direct_check_xrun(dshare, pcm);
+	if (err < 0)
+		return err;
 
-	return snd_pcm_dshare_sync_ptr0(pcm, *dshare->spcm->hw.ptr);
+	return snd_pcm_dshare_sync_ptr0(pcm, slave_hw_ptr);
 }
 
 /*
@@ -237,12 +235,13 @@ static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 	case SNDRV_PCM_STATE_DRAINING:
 	case SNDRV_PCM_STATE_RUNNING:
 		snd_pcm_dshare_sync_ptr0(pcm, status->hw_ptr);
-		status->delay += snd_pcm_mmap_playback_delay(pcm);
+		status->delay = snd_pcm_mmap_playback_delay(pcm);
 		break;
 	default:
 		break;
 	}
 	status->state = snd_pcm_dshare_state(pcm);
+	status->hw_ptr = *pcm->hw.ptr; /* boundary may be different */
 	status->appl_ptr = *pcm->appl.ptr; /* slave PCM doesn't set appl_ptr */
 	status->trigger_tstamp = dshare->trigger_tstamp;
 	status->avail = snd_pcm_mmap_playback_avail(pcm);
@@ -254,22 +253,8 @@ static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 static snd_pcm_state_t snd_pcm_dshare_state(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
-	int err;
-	snd_pcm_state_t state;
-	state = snd_pcm_state(dshare->spcm);
-	switch (state) {
-	case SND_PCM_STATE_SUSPENDED:
-	case SND_PCM_STATE_DISCONNECTED:
-		dshare->state = state;
-		return state;
-	case SND_PCM_STATE_XRUN:
-		if ((err = snd_pcm_direct_slave_recover(dshare)) < 0)
-			return err;
-		break;
-	default:
-		break;
-	}
-	snd_pcm_direct_client_chk_xrun(dshare, pcm);
+
+	snd_pcm_direct_check_xrun(dshare, pcm);
 	if (dshare->state == STATE_RUN_PENDING)
 		return SNDRV_PCM_STATE_RUNNING;
 	return dshare->state;
@@ -279,7 +264,7 @@ static int snd_pcm_dshare_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	int err;
-	
+
 	switch (dshare->state) {
 	case SNDRV_PCM_STATE_DRAINING:
 	case SNDRV_PCM_STATE_RUNNING:
@@ -326,8 +311,7 @@ static int snd_pcm_dshare_reset(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	dshare->hw_ptr %= pcm->period_size;
 	dshare->appl_ptr = dshare->last_appl_ptr = dshare->hw_ptr;
-	dshare->slave_appl_ptr = dshare->slave_hw_ptr = *dshare->spcm->hw.ptr;
-	snd_pcm_direct_reset_slave_ptr(pcm, dshare);
+	snd_pcm_direct_reset_slave_ptr(pcm, dshare, *dshare->spcm->hw.ptr);
 	return 0;
 }
 
@@ -336,8 +320,7 @@ static int snd_pcm_dshare_start_timer(snd_pcm_t *pcm, snd_pcm_direct_t *dshare)
 	int err;
 
 	snd_pcm_hwsync(dshare->spcm);
-	dshare->slave_appl_ptr = dshare->slave_hw_ptr = *dshare->spcm->hw.ptr;
-	snd_pcm_direct_reset_slave_ptr(pcm, dshare);
+	snd_pcm_direct_reset_slave_ptr(pcm, dshare, *dshare->spcm->hw.ptr);
 	err = snd_timer_start(dshare->timer);
 	if (err < 0)
 		return err;
@@ -350,7 +333,7 @@ static int snd_pcm_dshare_start(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	snd_pcm_sframes_t avail;
 	int err;
-	
+
 	if (dshare->state != SND_PCM_STATE_PREPARED)
 		return -EBADFD;
 	avail = snd_pcm_mmap_playback_hw_avail(pcm);
@@ -423,7 +406,7 @@ static int __snd_pcm_dshare_drain(snd_pcm_t *pcm)
 		}
 		if (dshare->state == SND_PCM_STATE_DRAINING) {
 			snd_pcm_dshare_sync_area(pcm);
-			snd_pcm_wait_nocheck(pcm, -1);
+			snd_pcm_wait_nocheck(pcm, SND_PCM_WAIT_DRAIN);
 			snd_pcm_direct_clear_timer_queue(dshare); /* force poll to wait */
 
 			switch (snd_pcm_state(dshare->spcm)) {
@@ -506,10 +489,10 @@ static int snd_pcm_dshare_close(snd_pcm_t *pcm)
 	snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
 	dshare->shmptr->u.dshare.chn_mask &= ~dshare->u.dshare.chn_mask;
 	snd_pcm_close(dshare->spcm);
- 	if (dshare->server)
- 		snd_pcm_direct_server_discard(dshare);
- 	if (dshare->client)
- 		snd_pcm_direct_client_discard(dshare);
+	if (dshare->server)
+		snd_pcm_direct_server_discard(dshare);
+	if (dshare->client)
+		snd_pcm_direct_client_discard(dshare);
 	if (snd_pcm_direct_shm_discard(dshare)) {
 		if (snd_pcm_direct_semaphore_discard(dshare))
 			snd_pcm_direct_semaphore_final(dshare, DIRECT_IPC_SEM_CLIENT);
@@ -528,18 +511,9 @@ static snd_pcm_sframes_t snd_pcm_dshare_mmap_commit(snd_pcm_t *pcm,
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	int err;
 
-	switch (snd_pcm_state(dshare->spcm)) {
-	case SND_PCM_STATE_XRUN:
-		if ((err = snd_pcm_direct_slave_recover(dshare)) < 0)
-			return err;
-		break;
-	case SND_PCM_STATE_SUSPENDED:
-		return -ESTRPIPE;
-	default:
-		break;
-	}
-	if (snd_pcm_direct_client_chk_xrun(dshare, pcm))
-		return -EPIPE;
+	err = snd_pcm_direct_check_xrun(dshare, pcm);
+	if (err < 0)
+		return err;
 	if (! size)
 		return 0;
 	snd_pcm_mmap_appl_forward(pcm, size);
@@ -568,7 +542,7 @@ static snd_pcm_sframes_t snd_pcm_dshare_avail_update(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	int err;
-	
+
 	if (dshare->state == SND_PCM_STATE_RUNNING ||
 	    dshare->state == SND_PCM_STATE_DRAINING) {
 		if ((err = snd_pcm_dshare_sync_ptr(pcm)) < 0)
@@ -587,7 +561,7 @@ static int snd_pcm_dshare_htimestamp(snd_pcm_t *pcm,
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	snd_pcm_uframes_t avail1;
 	int ok = 0;
-	
+
 	while (1) {
 		if (dshare->state == SND_PCM_STATE_RUNNING ||
 		    dshare->state == SND_PCM_STATE_DRAINING)
@@ -698,7 +672,7 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	assert(pcmp);
 
 	if (stream != SND_PCM_STREAM_PLAYBACK) {
-		SNDERR("The dshare plugin supports only playback stream");
+		snd_error(PCM, "The dshare plugin supports only playback stream");
 		return -EINVAL;
 	}
 
@@ -729,27 +703,27 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 		ret = snd_pcm_open_slave(&spcm, root, sconf, stream,
 					 mode | SND_PCM_NONBLOCK, NULL);
 		if (ret < 0) {
-			SNDERR("unable to open slave");
+			snd_error(PCM, "unable to open slave");
 			goto _err;
 		}
-	
+
 		if (snd_pcm_type(spcm) != SND_PCM_TYPE_HW) {
-			SNDERR("dshare plugin can be only connected to hw plugin");
+			snd_error(PCM, "dshare plugin can be only connected to hw plugin");
 			goto _err;
 		}
-		
+
 		ret = snd_pcm_direct_initialize_slave(dshare, spcm, params);
 		if (ret < 0) {
-			SNDERR("unable to initialize slave");
+			snd_error(PCM, "unable to initialize slave");
 			goto _err;
 		}
 
 		dshare->spcm = spcm;
-		
+
 		if (dshare->shmptr->use_server) {
 			ret = snd_pcm_direct_server_create(dshare);
 			if (ret < 0) {
-				SNDERR("unable to create server");
+				snd_error(PCM, "unable to create server");
 				goto _err;
 			}
 		}
@@ -761,10 +735,10 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 			snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
 			ret = snd_pcm_direct_client_connect(dshare);
 			if (ret < 0) {
-				SNDERR("unable to connect client");
+				snd_error(PCM, "unable to connect client");
 				goto _err_nosem;
 			}
-			
+
 			snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
 			ret = snd_pcm_direct_open_secondary_client(&spcm, dshare, "dshare_client");
 			if (ret < 0)
@@ -784,18 +758,18 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 					first_instance = 1;
 					goto retry;
 				}
-				SNDERR("unable to open slave");
+				snd_error(PCM, "unable to open slave");
 				goto _err;
 			}
 			if (snd_pcm_type(spcm) != SND_PCM_TYPE_HW) {
-				SNDERR("dshare plugin can be only connected to hw plugin");
+				snd_error(PCM, "dshare plugin can be only connected to hw plugin");
 				ret = -EINVAL;
 				goto _err;
 			}
-		
+
 			ret = snd_pcm_direct_initialize_secondary_slave(dshare, spcm, params);
 			if (ret < 0) {
-				SNDERR("unable to initialize slave");
+				snd_error(PCM, "unable to initialize slave");
 				goto _err;
 			}
 		}
@@ -809,16 +783,16 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 			dshare->u.dshare.chn_mask |= (1ULL << dchn);
 	}
 	if (dshare->shmptr->u.dshare.chn_mask & dshare->u.dshare.chn_mask) {
-		SNDERR("destination channel specified in bindings is already used");
+		snd_error(PCM, "destination channel specified in bindings is already used");
 		dshare->u.dshare.chn_mask = 0;
 		ret = -EINVAL;
 		goto _err;
 	}
 	dshare->shmptr->u.dshare.chn_mask |= dshare->u.dshare.chn_mask;
-		
+
 	ret = snd_pcm_direct_initialize_poll_fd(dshare);
 	if (ret < 0) {
-		SNDERR("unable to initialize poll_fd");
+		snd_error(PCM, "unable to initialize poll_fd");
 		goto _err;
 	}
 
@@ -828,12 +802,12 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	pcm->mmap_rw = 1;
 	snd_pcm_set_hw_ptr(pcm, &dshare->hw_ptr, -1, 0);
 	snd_pcm_set_appl_ptr(pcm, &dshare->appl_ptr, -1, 0);
-	
+
 	snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
 
 	*pcmp = pcm;
 	return 0;
-	
+
  _err:
 	if (dshare->shmptr != (void *) -1)
 		dshare->shmptr->u.dshare.chn_mask &= ~dshare->u.dshare.chn_mask;
@@ -876,7 +850,7 @@ pcm.name {
 	ipc_perm INT		# IPC permissions (octal, default 0600)
 	hw_ptr_alignment STR	# Slave application and hw pointer alignment type
 		# STR can be one of the below strings :
-		# no
+		# no (or off)
 		# roundup
 		# rounddown
 		# auto (default)
